### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from z3c.form.interfaces import IWidget, IFieldWidget, IMultiWidget, ITextWidget, IFormLayer, NO_VALUE
from zope.interface.common.idatetime import ITZInfo
from zope.schema.interfaces import IField, IDatetime

# import local interfaces
from ztfy.skin.interfaces import IDateWidget, IDatetimeWidget, IFixedWidthTextAreaWidget
from ztfy.skin.layer import IZTFYBrowserLayer
from ztfy.utils.schema import IDatesRangeField, IColorField, ITextLineListField

# import Zope3 packages
from z3c.form.browser.text import TextWidget
from z3c.form.browser.textarea import TextAreaWidget
from z3c.form.browser.widget import HTMLFormElement
from z3c.form.converter import DatetimeDataConverter as BaseDatetimeDataConverter, SequenceDataConverter, FormatterValidationError
from z3c.form.widget import FieldWidget, Widget
from zope.component import adapter, adapts
from zope.i18n.format import DateTimeParseError
from zope.interface import implementer, implements, implementsOnly

# import local packages
from ztfy.jqueryui import jquery_datetime, jquery_colorpicker
from ztfy.skin import ztfy_skin_base


#
# Date and DateTime widgets
#

class DateWidget(TextWidget):
    """Date input widget"""
    implementsOnly(IDateWidget)

    @property
    def pattern(self):
        result = self.request.locale.dates.getFormatter('date', 'short').getPattern()
        return result.replace('d', '%d') \
                     .replace('dd', '%d') \
                     .replace('%d%d', '%d') \
                     .replace('MM', '%m') \
                     .replace('M', '%m') \
                     .replace('yy', '%y')

    def render(self):
        result = super(DateWidget, self).render()
        if result:
            ztfy_skin_base.need()
            jquery_datetime.need()
        return result


@adapter(IField, IFormLayer)
@implementer(IFieldWidget)
def DateFieldWidget(field, request):
    """IFieldWidget factory for DateField"""
    return FieldWidget(field, DateWidget(request))


class DatetimeWidget(TextWidget):
    """Datetime input widget"""
    implementsOnly(IDatetimeWidget)

    @property
    def pattern(self):
        result = self.request.locale.dates.getFormatter('dateTime', 'short').getPattern()
        return result.replace('d', '%d') \
                     .replace('dd', '%d') \
                     .replace('%d%d', '%d') \
                     .replace('MM', '%m') \
                     .replace('M', '%m') \
                     .replace('yy', '%y') \
                     .replace('HH', '%H') \
                     .replace('h', '%H') \
                     .replace('mm', '%M') \
                     .replace('a', '%p')

    def render(self):
        result = super(DatetimeWidget, self).render()
        if result:
            ztfy_skin_base.need()
            jquery_datetime.need()
        return result


@adapter(IField, IFormLayer)
@implementer(IFieldWidget)
def DatetimeFieldWidget(field, request):
    """IFieldWidget factory for DatetimeField"""
    return FieldWidget(field, DatetimeWidget(request))


class DatetimeDataConverter(BaseDatetimeDataConverter):

    adapts(IDatetime, IDatetimeWidget)

    def toFieldValue(self, value):
        value = super(DatetimeDataConverter, self).toFieldValue(value)
        if value and not value.tzinfo:
            tz = ITZInfo(self.widget.request)
            value = tz.localize(value)
        return value


#
# Dates range widget
#

class IDatesRangeWidget(IMultiWidget):
    """Dates range widget interface"""


class DatesRangeDataConverter(SequenceDataConverter):
    """Dates range data converter"""

    adapts(IDatesRangeField, IDatesRangeWidget)

    def toWidgetValue(self, value):
        if value is self.field.missing_value:
            return (u'', u'')
        locale = self.widget.request.locale
        formatter = locale.dates.getFormatter('date', 'short')
        return (formatter.format(value[0]) if value[0] else None,
                formatter.format(value[1]) if value[1] else None)

    def toFieldValue(self, value):
        if not value:
            return self.field.missing_value
        try:
            locale = self.widget.request.locale
            formatter = locale.dates.getFormatter('date', 'short')
            return (formatter.parse(value[0]) if value[0] else None,
                    formatter.parse(value[1]) if value[1] else None)
        except DateTimeParseError, err:
            raise FormatterValidationError(err.args[0], value)


class DatesRangeWidget(HTMLFormElement, Widget):
    """Dates range widget"""

    implements(IDatesRangeWidget)

    @property
    def pattern(self):
        result = self.request.locale.dates.getFormatter('date', 'short').getPattern()
        return result.replace('dd', '%d').replace('MM', '%m').replace('yy', '%y')

    @property
    def begin_id(self):
        return '%s-begin' % self.id

    @property
    def begin_name(self):
        return '%s.begin' % self.name

    @property
    def begin_date(self):
        return (self.value[0] or '') if self.value else ''

    @property
    def end_id(self):
        return '%s-end' % self.id

    @property
    def end_name(self):
        return '%s.end' % self.name

    @property
    def end_date(self):
        return (self.value[1] or '') if self.value else ''

    def extract(self, default=NO_VALUE):
        begin_date = self.request.get(self.begin_name)
        end_date = self.request.get(self.end_name)
        return (begin_date, end_date)

    def render(self):
        result = super(DatesRangeWidget, self).render()
        if result:
            ztfy_skin_base.need()
            jquery_datetime.need()
        return result


@adapter(IDatesRangeField, IFormLayer)
@implementer(IFieldWidget)
def DatesRangeFieldWidgetFactory(field, request):
    """IDatesRangeField widget factory"""
    return FieldWidget(field, DatesRangeWidget(request))


#
# Color widget
#

class IColorWidget(ITextWidget):
    """Color widget interface"""


class ColorWidget(TextWidget):
    """Color widget"""

    implementsOnly(IColorWidget)

    def render(self):
        result = super(ColorWidget, self).render()
        if result:
            ztfy_skin_base.need()
            jquery_colorpicker.need()
        return result


@adapter(IColorField, IFormLayer)
@implementer(IFieldWidget)
def ColorFieldWidgetFactory(field, request):
    """IColorField widget factory"""
    return FieldWidget(field, ColorWidget(request))


#
# Textline list widget
#

class ITextLineListWidget(ITextWidget):
    """TextLineList widget interface"""


class TextLineListDataConverter(SequenceDataConverter):
    """TextLineList field data converter"""

    adapts(ITextLineListField, IWidget)

    def toWidgetValue(self, value):
        if value is self.field.missing_value:
            return u''
        return '|'.join(value)

    def toFieldValue(self, value):
        if not value:
            return self.field.missing_value
        return value.split('|')


class TextLineListWidget(TextWidget):
    """TextLineList field widget"""

    implementsOnly(ITextLineListWidget)

    def render(self):
        result = super(TextLineListWidget, self).render()
        if result:
            ztfy_skin_base.need()
        return result


@adapter(ITextLineListField, IFormLayer)
@implementer(IFieldWidget)
def TextLineListFieldWidgetFactory(field, request):
    return FieldWidget(field, TextLineListWidget(request))


#
# Fixed width text area widget
#

class FixedWidthTextAreaWidget(TextAreaWidget):
    """Custom fixed width text area widget"""
    implementsOnly(IFixedWidthTextAreaWidget)


@adapter(IField, IZTFYBrowserLayer)
@implementer(IFieldWidget)
def FixedWidthTextAreaFieldWidget(field, request):
    """IFixedWidthTextAreaWidget factory"""
    return FieldWidget(field, FixedWidthTextAreaWidget(request))



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from z3c.language.switch.interfaces import II18n
from zope.dublincore.interfaces import IZopeDublinCore

# import local interfaces
from ztfy.skin.interfaces import IDefaultView, IBreadcrumbInfo

# import Zope3 packages
from zope.component import queryMultiAdapter
from zope.traversing.api import getParents
from zope.traversing.browser import absoluteURL

# import local packages
from ztfy.skin.viewlet import ViewletBase


class BreadcrumbsViewlet(ViewletBase):

    viewname = ''

    @property
    def crumbs(self):
        result = []
        for parent in reversed([self.context, ] + getParents(self.context)):
            info = queryMultiAdapter((parent, self.request, self.__parent__), IBreadcrumbInfo)
            if info is not None:
                result.append({ 'title': info.title,
                                'path': info.path,
                                'class': '' })
            else:
                i18n = II18n(parent, None)
                if i18n is not None:
                    name = i18n.queryAttribute('shortname', request=self.request) or i18n.queryAttribute('title', request=self.request)
                else:
                    dc = IZopeDublinCore(parent, None)
                    if dc is not None:
                        name = dc.title
                if name:
                    adapter = queryMultiAdapter((parent, self.request, self.__parent__), IDefaultView)
                    if (adapter is not None) and adapter.viewname:
                        self.viewname = '/' + adapter.viewname
                    result.append({ 'title': name,
                                    'path': '%s%s' % (absoluteURL(parent, request=self.request),
                                                      self.viewname),
                                    'class': '' })
        if result:
            result[-1]['class'] = 'current'
        return result



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from z3c.form.interfaces import HIDDEN_MODE
from z3c.json.interfaces import IJSONWriter
from zope.authentication.interfaces import IAuthentication, ILogout
from zope.component.interfaces import ISite
from zope.security.interfaces import IUnauthorized

# import local interfaces
from ztfy.skin.interfaces import IDefaultView, ILoginFormFields

# import Zope3 packages
from z3c.form import field, button
from z3c.formjs import ajax
from zope.component import  adapts, queryMultiAdapter, getUtility, getUtilitiesFor, hooks
from zope.interface import implements
from zope.traversing.browser.absoluteurl import absoluteURL

# import local packages
from ztfy.skin.form import AddForm
from ztfy.utils.traversing import getParent

from ztfy.skin import _


class LogoutAdapter(object):

    adapts(IAuthentication)
    implements(ILogout)

    def __init__(self, auth):
        self.auth = auth

    def logout(self, request):
        return self.auth.logout(request)


class LoginLogoutView(ajax.AJAXRequestHandler):
    """Base login/logout view"""

    @ajax.handler
    def login(self):
        writer = getUtility(IJSONWriter)
        context = getParent(self.context, ISite)
        while context is not None:
            old_site = hooks.getSite()
            try:
                hooks.setSite(context)
                for _name, auth in getUtilitiesFor(IAuthentication):
                    if auth.authenticate(self.request) is not None:
                        return writer.write('OK')
            finally:
                hooks.setSite(old_site)
            context = getParent(context, ISite, allow_context=False)
        return writer.write('NOK')

    @ajax.handler
    def logout(self):
        writer = getUtility(IJSONWriter)
        context = getParent(self.context, ISite)
        while context is not None:
            old_site = hooks.getSite()
            try:
                hooks.setSite(context)
                for _name, auth in getUtilitiesFor(IAuthentication):
                    if auth.logout(self.request):
                        return writer.write('OK')
            finally:
                hooks.setSite(old_site)
            context = getParent(context, ISite, allow_context=False)
        return writer.write('NOK')


class LoginForm(AddForm):
    """ZMI login form"""

    title = _("Login form")
    legend = _("Please enter valid credentials to login")

    fields = field.Fields(ILoginFormFields)

    def __call__(self):
        self.request.response.setStatus(401)
        return super(LoginForm, self).__call__()

    def updateWidgets(self):
        super(LoginForm, self).updateWidgets()
        self.widgets['came_from'].mode = HIDDEN_MODE
        if IUnauthorized.providedBy(self.context):
            self.widgets['came_from'].value = self.request.getURL()

    @button.buttonAndHandler(_("login-button", "Login"))
    def handleLogin(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        self.request.form['login'] = data['username']
        self.request.form['password'] = data['password']
        if IUnauthorized.providedBy(self.context):
            context, _layer, _permission = self.context.args
        else:
            context = self.context
        site = getParent(context, ISite)
        while site is not None:
            old_site = hooks.getSite()
            try:
                hooks.setSite(site)
                for _name, auth in getUtilitiesFor(IAuthentication):
                    if auth.authenticate(self.request) is not None:
                        target = data.get('came_from')
                        if target:
                            self.request.response.redirect(target)
                        else:
                            target = queryMultiAdapter((context, self.request, self), IDefaultView)
                            if target is not None:
                                self.request.response.redirect(target.getAbsoluteURL())
                            else:
                                self.request.response.redirect('%s/@@SelectedManagementView.html' % absoluteURL(self.context, self.request))
                        return u''
            finally:
                hooks.setSite(old_site)
            site = getParent(site, ISite, allow_context=False)



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

"""
This module contains ztfy.skin package
"""
import os
from setuptools import setup, find_packages

DOCS = os.path.join(os.path.dirname(__file__),
                    'docs')

README = os.path.join(DOCS, 'README.txt')
HISTORY = os.path.join(DOCS, 'HISTORY.txt')

version = '0.5.19'
long_description = open(README).read() + '\n\n' + open(HISTORY).read()

tests_require = [
        'zope.testing',
    ]

setup(name='ztfy.skin',
      version=version,
      description="ZTFY skin utility package, including views, forms and viewlets base classes",
      long_description=long_description,
      classifiers=[
          "License :: OSI Approved :: Zope Public License",
          "Development Status :: 4 - Beta",
          "Programming Language :: Python",
          "Framework :: Zope3",
          "Topic :: Software Development :: Libraries :: Python Modules",
      ],
      keywords='ZTFY Zope3 skin',
      author='Thierry Florac',
      author_email='tflorac@ulthar.net',
      url='http://www.ztfy.org',
      license='ZPL',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['ztfy'],
      include_package_data=True,
      package_data={'': ['*.zcml', '*.txt', '*.pt', '*.pot', '*.po', '*.mo', '*.png', '*.gif', '*.jpeg', '*.jpg', '*.css', '*.js']},
      zip_safe=False,
      # uncomment this to be able to run tests with setup.py
      #test_suite = "ztfy.skin.tests.test_skindocs.test_suite",
      tests_require=tests_require,
      extras_require=dict(test=tests_require),
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'fanstatic',
          'jquery.layer',
          'z3c.form',
          'z3c.formjs',
          'z3c.formui',
          'z3c.json',
          'z3c.jsonrpc',
          'z3c.language.switch',
          'z3c.layer.pagelet',
          'z3c.menu.simple',
          'z3c.table',
          'z3c.template',
          'zope.app.renderer',
          'zope.authentication',
          'zope.browserpage',
          'zope.browserresource',
          'zope.component',
          'zope.container',
          'zope.contentprovider',
          'zope.dublincore',
          'zope.event',
          'zope.i18n',
          'zope.i18nmessageid',
          'zope.interface',
          'zope.intid',
          'zope.lifecycleevent',
          'zope.publisher',
          'zope.schema',
          'zope.security',
          'zope.traversing',
          'zope.viewlet',
          'ztfy.jqueryui >= 0.7.0',
          'ztfy.utils >= 0.4.0',
      ],
      entry_points={
          'fanstatic.libraries': [
              'ztfy.skin = ztfy.skin:library',
          ]
      })



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2012 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from z3c.language.switch.interfaces import II18n
from z3c.template.interfaces import IPageTemplate
from zope.dublincore.interfaces import IZopeDublinCore

# import local interfaces
from ztfy.skin.interfaces import IContainerBaseView, IOrderedContainerBaseView, \
                                 IContainerAddFormMenuTarget, \
                                 IOrderedContainerSorterColumn, IOrderedContainer

# import Zope3 packages
from z3c.formjs import ajax
from z3c.table.batch import BatchProvider
from z3c.table.column import Column
from z3c.table.table import Table
from z3c.template.template import getViewTemplate, getPageTemplate
from zope.i18n import translate
from zope.component import getMultiAdapter
from zope.interface import implements
from zope.traversing.api import getName

# import local packages
from ztfy.jqueryui import jquery_ui_css, jquery_jsonrpc, jquery_ui_base, jquery_tipsy
from ztfy.skin.menu import MenuItem
from ztfy.skin.page import BaseBackView, TemplateBasedPage

from ztfy.skin import _


class ContainerContentsViewMenu(MenuItem):
    """Container contents menu"""

    title = _("Contents")


class OrderedContainerSorterColumn(Column):

    implements(IOrderedContainerSorterColumn)

    header = u''
    weight = 1
    cssClasses = { 'th': 'sorter',
                   'td': 'sorter' }

    def renderCell(self, item):
        return '<img class="handler" src="/--static--/ztfy.skin/img/sort.png" />'


class ContainerBatchProvider(BatchProvider):
    """Custom container batch provider"""

    batchSpacer = u'<a>...</a>'


class BaseContainerBaseView(TemplateBasedPage, Table):
    """Container-like base view"""

    implements(IContainerBaseView)

    legend = _("Container's content")
    empty_message = _("This container is empty")

    data_attributes = {}

    batchSize = 25
    startBatchingAt = 25

    output = getViewTemplate()

    def __init__(self, context, request):
        super(BaseContainerBaseView, self).__init__(context, request)
        Table.__init__(self, context, request)

    @property
    def title(self):
        result = None
        i18n = II18n(self.context, None)
        if i18n is not None:
            result = II18n(self.context).queryAttribute('title', request=self.request)
        if result is None:
            dc = IZopeDublinCore(self.context, None)
            if dc is not None:
                result = dc.title
        if not result:
            result = '{{ %s }}' % getName(self.context)
        return result

    def getCSSClass(self, element, cssClass=None):
        result = super(BaseContainerBaseView, self).getCSSClass(element, cssClass)
        result += ' ' + ' '.join(('data-%s=%s' % (k, v) for k, v in self.data_attributes.items()))
        return result

    @property
    def empty_value(self):
        return translate(self.empty_message, context=self.request)

    def update(self):
        TemplateBasedPage.update(self)
        Table.update(self)
        jquery_tipsy.need()
        jquery_ui_css.need()
        jquery_jsonrpc.need()


class ContainerBaseView(BaseBackView, BaseContainerBaseView):
    """Back-office container base view"""

    def update(self):
        BaseBackView.update(self)
        BaseContainerBaseView.update(self)


class OrderedContainerBaseView(ajax.AJAXRequestHandler, ContainerBaseView):

    implements(IOrderedContainerBaseView, IContainerAddFormMenuTarget)

    sortOn = None
    interface = None
    container_interface = IOrderedContainer

    batchSize = 10000
    startBatchingAt = 10000

    cssClasses = { 'table': 'orderable' }

    def update(self):
        super(OrderedContainerBaseView, self).update()
        jquery_ui_base.need()
        jquery_jsonrpc.need()

    @ajax.handler
    def ajaxUpdateOrder(self, *args, **kw):
        self.updateOrder()

    def updateOrder(self, context=None):
        ids = self.request.form.get('ids', [])
        if ids:
            if context is None:
                context = self.context
            container = self.container_interface(context)
            container.updateOrder(ids, self.interface)


class InnerContainerBaseView(Table):
    """A container table displayed inside another view"""

    template = getPageTemplate()

    data_attributes = {}

    def __init__(self, view, request):
        Table.__init__(self, view.context, request)
        self.__parent__ = view
        self.__name__ = u''

    def getCSSClass(self, element, cssClass=None):
        result = super(InnerContainerBaseView, self).getCSSClass(element, cssClass)
        result += ' ' + ' '.join(('data-%s=%s' % (k, v) for k, v in self.data_attributes.items()))
        return result

    def render(self):
        if self.template is None:
            template = getMultiAdapter((self, self.request), IPageTemplate)
            return template(self)
        return self.template()



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from z3c.form.interfaces import ISubForm, IWidget, ITextWidget, ITextAreaWidget, INPUT_MODE
from zope.container.interfaces import IContainer
from zope.lifecycleevent.interfaces import IObjectCreatedEvent
from zope.viewlet.interfaces import IViewlet, IViewletManager

# import local interfaces

# import Zope3 packages
from z3c.form import button
from z3c.formjs import jsaction
from zope.interface import Attribute, Interface
from zope.schema import Bool, TextLine, List, Choice, Dict, Object, Password

# import local packages

from ztfy.skin import _


#
# Skinnable contents
#

class ISkinnable(Interface):
    """Base skinnable contents interface"""

    def getSkin(self):
        """Get real skin applied to context"""


#
# Custom widgets interfaces
#

class IDateWidget(ITextWidget):
    """Marker interface for date widget"""


class IDatetimeWidget(ITextWidget):
    """Marker interface for datetime widget"""


class IFixedWidthTextAreaWidget(ITextAreaWidget):
    """Marker interface for fixed width text area widget"""


#
# Default views interfaces
#

class IDefaultView(Interface):
    """Interface used to get object's default view name"""

    viewname = TextLine(title=_("View name"),
                        description=_("Name of the default view for the adapter object and request"),
                        required=True,
                        default=u'@@index.html')

    def getAbsoluteURL(self):
        """Get full absolute URL of the default view"""


class IContainedDefaultView(IDefaultView):
    """Interface used to get object's default view name while displayed inside a container"""


class IContainerAddFormMenuTarget(Interface):
    """Marker interface for base add form menu item"""


class IPropertiesMenuTarget(Interface):
    """Marker interface for properties menus"""


#
# Containers interfaces
#

class IOrderedContainerOrder(Interface):
    """Ordered containers interface"""

    def updateOrder(self, order):
        """Reset items in given order"""

    def moveUp(self, key):
        """Move given item up"""

    def moveDown(self, key):
        """Move given item down"""

    def moveFirst(self, key):
        """Move given item to first position"""

    def moveLast(self, key):
        """Move given item to last position"""


class IOrderedContainer(IOrderedContainerOrder, IContainer):
    """Marker interface for ordered containers"""


class IContainerBaseView(Interface):
    """Marker interface for container base view"""


class IOrderedContainerBaseView(Interface):
    """Marker interface for ordered container based view"""


class IOrderedContainerSorterColumn(Interface):
    """Marker interface for container sorter column"""


#
# Custom events interfaces
#

class IFormObjectCreatedEvent(IObjectCreatedEvent):
    """Object added event notify by form after final object creation"""

    view = Attribute(_("Event source form view"))


#
# Custom forms interfaces
#

def checkSubmitButton(form):
    """Check form and widgets mode before displaying submit button"""
    if form.mode != INPUT_MODE:
        return False
    for widget in form.widgets.values():
        if widget.mode == INPUT_MODE:
            return True
    if IForm.providedBy(form):
        for subform in form.subforms:
            for widget in subform.widgets.values():
                if widget.mode == INPUT_MODE:
                    return True


class IEditFormButtons(Interface):
    """Default edit form buttons"""

    submit = button.Button(title=_("Submit"), condition=checkSubmitButton)
    reset = button.Button(title=_("Reset"))


class IDialogAddFormButtons(Interface):
    """Default dialog add form buttons"""

    add = jsaction.JSButton(title=_("Add"))
    cancel = jsaction.JSButton(title=_("Cancel"))


class IDialogDisplayFormButtons(Interface):
    """Default dialog display form buttons"""

    dialog_close = jsaction.JSButton(title=_("Close"))


class IDialogEditFormButtons(Interface):
    """Default dialog edit form buttons"""

    dialog_submit = jsaction.JSButton(title=_("Submit"), condition=checkSubmitButton)
    dialog_cancel = jsaction.JSButton(title=_("Cancel"))


class IWidgetsGroup(Interface):
    """Form widgets group interface"""

    id = Attribute(_("Group ID"))

    widgets = List(title=_("Group widgets"),
                   value_type=Object(IWidget))

    cssClass = Attribute(_("CSS class"))

    legend = Attribute(_("Group title"))

    help = Attribute(_("Group help"))

    switch = Bool(title=_("Display group switch ?"),
                  required=True,
                  default=False)

    checkbox_witch = Bool(title=_("Use checkbox as switcher instead of '+' sign"),
                          required=True,
                          default=False)

    checkbox_on = Bool(title=_("Is checkbox switcher enabled by default?"),
                       required=True,
                       default=False)

    checkbox_field = Attribute(_("Schema field matching checkbox switcher"))

    hide_if_empty = Bool(title=_("Hide group if empty ?"),
                         description=_("""If 'Yes', a switchable group containing only """
                                       """widgets with default values is hidden"""),
                         required=True,
                         default=False)

    visible = Attribute(_("Visible group"))


class IGroupsBasedForm(Interface):
    """Groups based form"""

    groups = Attribute(_("Form groups"))

    def addGroup(self, group):
        """Add given group to form"""


class IBaseForm(Interface):
    """Marker interface for any form"""


class ICustomExtractSubForm(ISubForm):
    """SubForm interface with custom extract method"""

    def extract(self):
        """Extract data and errors from input request"""


class ICustomUpdateSubForm(ISubForm):
    """SubForm interface with custom update method"""

    def updateContent(self, object, data):
        """Update custom content with given data"""


class IForm(Interface):
    """Base form interface"""

    title = TextLine(title=_("Form title"))

    legend = TextLine(title=_("Form legend"),
                      required=False)

    subforms = List(title=_("Sub-forms"),
                    value_type=Object(schema=ISubForm),
                    required=False)

    tabforms = List(title=_("Tab-forms"),
                    value_type=Object(schema=ISubForm),
                    required=False)

    autocomplete = Choice(title=_("Auto-complete"),
                          values=('on', 'off'),
                          default='on')

    display_hints_on_widgets = Bool(title=_("Display hints on input widgets?"),
                                    required=True,
                                    default=False)

    callbacks = Dict(title=_("Widgets callbacks"),
                     key_type=TextLine(),
                     value_type=TextLine())

    def createSubForms(self):
        """Initialize sub-forms"""

    def createTabForms(self):
        """Initialize tab-forms"""

    def getForms(self):
        """Get full list of forms"""

    def getWidgetCallback(self, widget):
        """Get submit callback associated with a given widget"""


class IViewletsBasedForm(IForm):
    """Viewlets based form interface"""

    managers = List(title=_("Names list of viewlets managers included in this form"),
                    value_type=TextLine(),
                    required=True)


class ISubFormViewlet(IViewlet):
    """Sub-form viewlet interface"""

    legend = Attribute(_("Sub-form legend"))

    switchable = Attribute(_("Can the subform be hidden ?"))

    visible = Attribute(_("Is the subform initially visible ?"))

    callbacks = Dict(title=_("Widgets callbacks"),
                     key_type=TextLine(),
                     value_type=TextLine())

    def getWidgetCallback(self, widget):
        """Get submit callback associated with a given widget"""


class IDialog(Interface):
    """Base interface for AJAX dialogs"""

    resources = Attribute(_("List of resources needed by this dialog"))


class IDialogMenu(Interface):
    """Dialog access menu interface"""

    target = Attribute(_("Target dialog class"))


class IDialogTitle(Interface):
    """Dialog title getter interface"""

    def getTitle(self):
        """Get dialog title"""


#
# Default form content providers
#

class IFormViewletsManager(IViewletManager):
    """Base forms viewlets manager interface"""


class IFormPrefixViewletsManager(IFormViewletsManager):
    """Form prefix viewlets manager interface"""


class IWidgetsPrefixViewletsManager(IFormViewletsManager):
    """Form widgets prefix viewlets manager interface"""


class IWidgetsSuffixViewletsManager(IFormViewletsManager):
    """Form widgets suffix viewlets manager interface"""


class IFormSuffixViewletsManager(IFormViewletsManager):
    """Form suffix viewlets manager interface"""


#
# Breadcrumb interfaces
#

class IBreadcrumbInfo(Interface):
    """Get custom breadcrumb info of a given context"""

    visible = Bool(title=_("Visible ?"),
                   required=True,
                   default=True)

    title = TextLine(title=_("Title"),
                     required=True)

    path = TextLine(title=_("Path"),
                    required=False)


#
# Back-office custom properties marker interface
#

class ICustomBackOfficeInfoTarget(Interface):
    """Marker interface for custom back-office properties"""

    back_interface = Attribute(_("Back-office properties custom interface"))


#
# Login form attributes
#

class ILoginFormFields(Interface):
    """Login form fields interface"""

    username = TextLine(title=_("login-field", "Login"),
                        required=True)

    password = Password(title=_("password-field", "Password"),
                        required=True)

    came_from = TextLine(title=_("came-from", "Original address"),
                         required=False)
